@@ -47,7 +47,7 @@ import Text.Pandoc.Scripting (ScriptingEngine (engineLoadCustom),
 import qualified Text.Pandoc.UTF8 as UTF8
 
 readUtf8File :: PandocMonad m => FilePath -> m T.Text
-readUtf8File = fmap UTF8.toText . readFileStrict
+readUtf8File fp = readFileStrict fp >>= toTextM fp
 
 -- | Settings specifying how document output should be produced.
 data OutputSettings m = OutputSettings
@@ -93,6 +93,9 @@ optToOutputSettings scriptingEngine opts = do
                      return (defaultOutputFlavor,Nothing)
                    Just f  -> return (f, Nothing)
 
+  when (format == "asciidoctor") $ do
+    report $ Deprecated "asciidoctor" "use asciidoc instead"
+
   let makeSandboxed pureWriter =
           let files = maybe id (:) (optReferenceDoc opts) .
                       maybe id (:) (optEpubMetadata opts) .
@@ -140,15 +143,17 @@ optToOutputSettings scriptingEngine opts = do
                  Just t -> (runWithDefaultPartials $ compileTemplate path t) >>=
                            templateOrThrow
       return (w, wexts, templ)
-    else do
-      tmpl <- processCustomTemplate (compileDefaultTemplate format)
+    else
       if optSandbox opts
-      then case runPure (getWriter flvrd) of
+      then do
+        tmpl <- processCustomTemplate (compileDefaultTemplate format)
+        case runPure (getWriter flvrd) of
              Right (w, wexts) -> return (makeSandboxed w, wexts, tmpl)
              Left e           -> throwError e
       else do
-           (w, wexts) <- getWriter flvrd
-           return (w, wexts, tmpl)
+        (w, wexts) <- getWriter flvrd
+        tmpl <- processCustomTemplate (compileDefaultTemplate format)
+        return (w, wexts, tmpl)
 
 
   let addSyntaxMap existingmap f = do
@@ -174,7 +179,7 @@ optToOutputSettings scriptingEngine opts = do
                          (ListVal $ v : map toVal vs) ctxMap
               Nothing -> M.insert k (toVal vs) ctxMap
 
-  let getTextContents fp = UTF8.toText . fst <$> fetchItem (T.pack fp)
+  let getTextContents fp = (fst <$> fetchItem (T.pack fp)) >>= toTextM fp
 
   let setFilesVariableM k fps ctx = do
         xs <- mapM getTextContents fps
@@ -209,8 +214,9 @@ optToOutputSettings scriptingEngine opts = do
     >>=
     (\vars ->  if format == "dzslides"
                   then do
-                      dztempl <- UTF8.toText <$> readDataFile
-                                   ("dzslides" </> "template.html")
+                      dztempl <-
+                        let fp = "dzslides" </> "template.html"
+                         in readDataFile fp >>= toTextM fp
                       let dzline = "<!-- {{{{ dzslides core"
                       let dzcore = T.unlines
                                  $ dropWhile (not . (dzline `T.isPrefixOf`))
