@@ -19,7 +19,11 @@ that it has omitted the construct.
 
 AsciiDoc:  <http://www.methods.co.nz/asciidoc/>
 -}
-module Text.Pandoc.Writers.AsciiDoc (writeAsciiDoc, writeAsciiDoctor) where
+module Text.Pandoc.Writers.AsciiDoc (
+  writeAsciiDoc,
+  writeAsciiDocLegacy,
+  writeAsciiDoctor
+  ) where
 import Control.Monad (foldM)
 import Control.Monad.State.Strict
     ( StateT, MonadState(get), gets, modify, evalStateT )
@@ -49,7 +53,7 @@ data WriterState = WriterState { defListMarker       :: Text
                                , bulletListLevel     :: Int
                                , intraword           :: Bool
                                , autoIds             :: Set.Set Text
-                               , asciidoctorVariant  :: Bool
+                               , legacy              :: Bool
                                , inList              :: Bool
                                , hasMath             :: Bool
                                -- |0 is no table
@@ -64,7 +68,7 @@ defaultWriterState = WriterState { defListMarker      = "::"
                                  , bulletListLevel    = 0
                                  , intraword          = False
                                  , autoIds            = Set.empty
-                                 , asciidoctorVariant = False
+                                 , legacy             = False
                                  , inList             = False
                                  , hasMath            = False
                                  , tableNestingLevel  = 0
@@ -75,11 +79,16 @@ writeAsciiDoc :: PandocMonad m => WriterOptions -> Pandoc -> m Text
 writeAsciiDoc opts document =
   evalStateT (pandocToAsciiDoc opts document) defaultWriterState
 
--- | Convert Pandoc to AsciiDoctor compatible AsciiDoc.
+{-# DEPRECATED writeAsciiDoctor "Use writeAsciiDoc instead" #-}
+-- | Deprecated synonym of 'writeAsciiDoc'.
 writeAsciiDoctor :: PandocMonad m => WriterOptions -> Pandoc -> m Text
-writeAsciiDoctor opts document =
+writeAsciiDoctor = writeAsciiDoc
+
+-- | Convert Pandoc to legacy AsciiDoc.
+writeAsciiDocLegacy :: PandocMonad m => WriterOptions -> Pandoc -> m Text
+writeAsciiDocLegacy opts document =
   evalStateT (pandocToAsciiDoc opts document)
-    defaultWriterState{ asciidoctorVariant = True }
+    defaultWriterState{ legacy = True }
 
 type ADW = StateT WriterState
 
@@ -101,7 +110,7 @@ pandocToAsciiDoc opts (Pandoc meta blocks) = do
                $ defField "toc"
                   (writerTableOfContents opts &&
                    isJust (writerTemplate opts))
-               $ defField "math" (hasMath st)
+               $ defField "math" (hasMath st && not (legacy st))
                $ defField "titleblock" titleblock metadata
   return $ render colwidth $
     case writerTemplate opts of
@@ -345,7 +354,9 @@ blockToAsciiDoc opts (Div (ident,classes,_) bs) = do
                      case bs of
                        (Div (_,["title"],_) ts : rest) -> (ts, rest)
                        _ -> ([], bs)
-             admonitionTitle <- if null titleBs
+             admonitionTitle <- if null titleBs ||
+                                   -- If title matches class, omit
+                                   (T.toLower (T.strip (stringify titleBs))) == l
                                    then return mempty
                                    else ("." <>) <$>
                                          blockListToAsciiDoc opts titleBs
@@ -364,10 +375,10 @@ bulletListItemToAsciiDoc :: PandocMonad m
 bulletListItemToAsciiDoc opts blocks = do
   lev <- gets bulletListLevel
   modify $ \s -> s{ bulletListLevel = lev + 1 }
-  isAsciidoctor <- gets asciidoctorVariant
-  let blocksWithTasks = if isAsciidoctor
-                          then (taskListItemToAsciiDoc blocks)
-                          else blocks
+  isLegacy <- gets legacy
+  let blocksWithTasks = if isLegacy
+                          then blocks
+                          else (taskListItemToAsciiDoc blocks)
   contents <- foldM (addBlock opts) empty blocksWithTasks
   modify $ \s -> s{ bulletListLevel = lev }
   let marker = text (replicate (lev + 1) '*')
@@ -517,7 +528,7 @@ inlineToAsciiDoc opts (Strong lst) = do
   return $ marker <> contents <> marker
 inlineToAsciiDoc opts (Strikeout lst) = do
   contents <- inlineListToAsciiDoc opts lst
-  return $ "[line-through]*" <> contents <> "*"
+  return $ "[line-through]#" <> contents <> "#"
 inlineToAsciiDoc opts (Superscript lst) = do
   contents <- inlineListToAsciiDoc opts lst
   return $ "^" <> contents <> "^"
@@ -526,38 +537,38 @@ inlineToAsciiDoc opts (Subscript lst) = do
   return $ "~" <> contents <> "~"
 inlineToAsciiDoc opts (SmallCaps lst) = inlineListToAsciiDoc opts lst
 inlineToAsciiDoc opts (Quoted qt lst) = do
-  isAsciidoctor <- gets asciidoctorVariant
+  isLegacy <- gets legacy
   inlineListToAsciiDoc opts $
     case qt of
       SingleQuote
-        | isAsciidoctor -> [Str "'`"] ++ lst ++ [Str "`'"]
-        | otherwise     -> [Str "`"] ++ lst ++ [Str "'"]
+        | isLegacy     -> [Str "`"] ++ lst ++ [Str "'"]
+        | otherwise    -> [Str "'`"] ++ lst ++ [Str "`'"]
       DoubleQuote
-        | isAsciidoctor -> [Str "\"`"] ++ lst ++ [Str "`\""]
-        | otherwise     -> [Str "``"] ++ lst ++ [Str "''"]
+        | isLegacy     -> [Str "``"] ++ lst ++ [Str "''"]
+        | otherwise    -> [Str "\"`"] ++ lst ++ [Str "`\""]
 inlineToAsciiDoc _ (Code _ str) = do
-  isAsciidoctor <- gets asciidoctorVariant
+  isLegacy <- gets legacy
   let escChar '`' = "\\'"
       escChar c   = T.singleton c
   let contents = literal (T.concatMap escChar str)
   return $
-    if isAsciidoctor
-       then text "`+" <> contents <> "+`"
-       else text "`"  <> contents <> "`"
+    if isLegacy
+       then text "`"  <> contents <> "`"
+       else text "`+" <> contents <> "+`"
 inlineToAsciiDoc _ (Str str) = escapeString str
 inlineToAsciiDoc _ (Math InlineMath str) = do
-  isAsciidoctor <- gets asciidoctorVariant
+  isLegacy <- gets legacy
   modify $ \st -> st{ hasMath = True }
-  let content = if isAsciidoctor
-                then literal str
-                else "$" <> literal str <> "$"
+  let content = if isLegacy
+                then "$" <> literal str <> "$"
+                else literal str
   return $ "latexmath:[" <> content <> "]"
 inlineToAsciiDoc _ (Math DisplayMath str) = do
-  isAsciidoctor <- gets asciidoctorVariant
+  isLegacy <- gets legacy
   modify $ \st -> st{ hasMath = True }
-  let content = if isAsciidoctor
-                then literal str
-                else "\\[" <> literal str <> "\\]"
+  let content = if isLegacy
+                   then "\\[" <> literal str <> "\\]"
+                   else literal str
   inlist <- gets inList
   let sepline = if inlist
                    then text "+"
