@@ -28,7 +28,7 @@ import Codec.Archive.Zip
       toEntry,
       Entry(eRelativePath) )
 import Control.Applicative ((<|>))
-import Control.Monad (MonadPlus(mplus), unless, when)
+import Control.Monad (MonadPlus(mplus), unless, when, foldM)
 import Control.Monad.Except (catchError, throwError)
 import Control.Monad.Reader
     ( asks, MonadReader(local), MonadTrans(lift), ReaderT(runReaderT) )
@@ -36,7 +36,8 @@ import Control.Monad.State.Strict ( StateT(runStateT), gets, modify )
 import qualified Data.ByteString.Lazy as BL
 import Data.Containers.ListUtils (nubOrd)
 import Data.Char (isSpace, isLetter)
-import Data.List (intercalate, isPrefixOf, isSuffixOf)
+import Data.List (intercalate, isPrefixOf, isSuffixOf, sortBy)
+import Data.Ord (comparing)
 import Data.String (fromString)
 import qualified Data.Map as M
 import Data.Maybe (fromMaybe, isNothing, mapMaybe, maybeToList, isJust)
@@ -63,6 +64,7 @@ import Text.Pandoc.ImageSize
 import Text.Pandoc.Logging
 import Text.Pandoc.MIME (extensionFromMimeType, getMimeType, getMimeTypeDef)
 import Text.Pandoc.Options
+import Text.Pandoc.Readers.Docx.Parse (extractTarget)
 import Text.Pandoc.Writers.Docx.StyleMap
 import Text.Pandoc.Writers.Docx.Table as Table
 import Text.Pandoc.Writers.Docx.Types
@@ -77,9 +79,63 @@ import Text.Pandoc.XML.Light as XML
 import Data.Generics (mkT, everywhere)
 import Text.Collate.Lang (renderLang, Lang(..))
 
+-- from wml.xsd EG_RPrBase
+rPrTagOrder :: M.Map Text Int
+rPrTagOrder =
+  M.fromList
+  (zip [ "rStyle"
+    , "rFonts"
+    , "b"
+    , "bCs"
+    , "i"
+    , "iCs"
+    , "caps"
+    , "smallCaps"
+    , "strike"
+    , "dstrike"
+    , "outline"
+    , "shadow"
+    , "emboss"
+    , "imprint"
+    , "noProof"
+    , "snapToGrid"
+    , "vanish"
+    , "webHidden"
+    , "color"
+    , "spacing"
+    , "w"
+    , "kern"
+    , "position"
+    , "sz"
+    , "szCs"
+    , "highlight"
+    , "u"
+    , "effect"
+    , "bdr"
+    , "shd"
+    , "fitText"
+    , "vertAlign"
+    , "rtl"
+    , "cs"
+    , "em"
+    , "lang"
+    , "eastAsianLayout"
+    , "specVanish"
+    , "oMath"
+    ] [0..])
+
+sortSquashed :: [Element] -> [Element]
+sortSquashed l =
+  sortBy (comparing tagIndex) l
+  where
+    tagIndex :: Element -> Int
+    tagIndex el =
+      fromMaybe 0 (M.lookup tag rPrTagOrder)
+      where tag = (qName . elName) el
+
 squashProps :: EnvProps -> [Element]
-squashProps (EnvProps Nothing es) = es
-squashProps (EnvProps (Just e) es) = e : es
+squashProps (EnvProps Nothing es) = sortSquashed es
+squashProps (EnvProps (Just e) es) = sortSquashed (e : es)
 
 renumIdMap :: Int -> [Element] -> M.Map Text Text
 renumIdMap _ [] = M.empty
@@ -124,6 +180,7 @@ writeDocx :: (PandocMonad m)
           -> m BL.ByteString
 writeDocx opts doc = do
   let Pandoc meta blocks = walk fixDisplayMath doc
+  setupTranslations meta
   let blocks' = makeSections True Nothing blocks
   let doc' = Pandoc meta blocks'
 
@@ -254,8 +311,6 @@ writeDocx opts doc = do
   let headers = filterElements isHeaderNode parsedRels
   let footers = filterElements isFooterNode parsedRels
 
-  let extractTarget = findAttr (QName "Target" Nothing Nothing)
-
   -- we create [Content_Types].xml and word/_rels/document.xml.rels
   -- from scratch rather than reading from reference.docx,
   -- because Word sometimes changes these files when a reference.docx is modified,
@@ -306,10 +361,13 @@ writeDocx opts doc = do
                         | e <- zEntries refArchive
                         , "word/media/" `isPrefixOf` eRelativePath e ]
 
-  let defaultnodes = [mknode "Default"
-              [("Extension","xml"),("ContentType","application/xml")] (),
-             mknode "Default"
-              [("Extension","rels"),("ContentType","application/vnd.openxmlformats-package.relationships+xml")] ()]
+  let mkDefaultNode (ext, mt) =
+        mknode "Default" [("Extension",ext),("ContentType",mt)] ()
+  let defaultnodes = map mkDefaultNode
+        [("xml", "application/xml"),
+         ("rels", "application/vnd.openxmlformats-package.relationships+xml"),
+         ("odttf",
+           "application/vnd.openxmlformats-officedocument.obfuscatedFont")]
   let contentTypesDoc = mknode "Types" [("xmlns","http://schemas.openxmlformats.org/package/2006/content-types")] $ defaultnodes ++ overrides
   let contentTypesEntry = toEntry "[Content_Types].xml" epochtime
         $ renderXml contentTypesDoc
@@ -504,29 +562,104 @@ writeDocx opts doc = do
   -- adds references to footnotes or endnotes we don't have...
   -- we do, however, copy some settings over from reference
   let settingsPath = "word/settings.xml"
-      settingsList = [ "zoom"
+
+  settingsEntry <- copyChildren refArchive distArchive settingsPath epochtime
+                      -- note: these must go in the following order:
+                     [ "writeProtection"
+                     , "view"
+                     , "zoom"
+                     , "removePersonalInformation"
+                     , "removeDateAndTime"
+                     , "doNotDisplayPageBoundaries"
+                     , "displayBackgroundShape"
+                     , "printPostScriptOverText"
+                     , "printFractionalCharacterWidth"
+                     , "printFormsData"
+                     , "embedTrueTypeFonts"
                      , "embedSystemFonts"
+                     , "saveSubsetFonts"
+                     , "saveFormsData"
+                     , "mirrorMargins"
+                     , "alignBordersAndEdges"
+                     , "bordersDoNotSurroundHeader"
+                     , "bordersDoNotSurroundFooter"
+                     , "gutterAtTop"
+                     , "hideSpellingErrors"
+                     , "hideGrammaticalErrors"
+                     , "activeWritingStyle"
+                     , "proofState"
+                     , "formsDesign"
+                     , "attachedTemplate"
+                     , "linkStyles"
+                     , "stylePaneFormatFilter"
+                     , "stylePaneSortMethod"
+                     , "documentType"
+                     , "mailMerge"
+                     , "revisionView"
+                     , "trackRevisions"
                      , "doNotTrackMoves"
+                     , "doNotTrackFormatting"
+                     , "documentProtection"
+                     , "autoFormatOverride"
+                     , "styleLockTheme"
+                     , "styleLockQFSet"
                      , "defaultTabStop"
+                     , "autoHyphenation"
+                     , "consecutiveHyphenLimit"
+                     , "hyphenationZone"
+                     , "doNotHyphenateCaps"
+                     , "showEnvelope"
+                     , "summaryLength"
+                     , "clickAndTypeStyle"
+                     , "defaultTableStyle"
+                     , "evenAndOddHeaders"
+                     , "bookFoldRevPrinting"
+                     , "bookFoldPrinting"
+                     , "bookFoldPrintingSheets"
                      , "drawingGridHorizontalSpacing"
                      , "drawingGridVerticalSpacing"
                      , "displayHorizontalDrawingGridEvery"
                      , "displayVerticalDrawingGridEvery"
+                     , "doNotUseMarginsForDrawingGridOrigin"
+                     , "drawingGridHorizontalOrigin"
+                     , "drawingGridVerticalOrigin"
+                     , "doNotShadeFormData"
+                     , "noPunctuationKerning"
                      , "characterSpacingControl"
+                     , "printTwoOnOne"
+                     , "strictFirstAndLastChars"
+                     , "noLineBreaksAfter"
+                     , "noLineBreaksBefore"
                      , "savePreviewPicture"
-                     , "mathPr"
-                     , "themeFontLang"
-                     , "decimalSymbol"
-                     , "listSeparator"
-                     , "autoHyphenation"
-                     , "consecutiveHyphenLimit"
-                     , "hyphenationZone"
-                     , "doNotHyphenateCap"
-                     , "evenAndOddHeaders"
-                     , "proofState"
+                     , "doNotValidateAgainstSchema"
+                     , "saveInvalidXml"
+                     , "ignoreMixedContent"
+                     , "alwaysShowPlaceholderText"
+                     , "doNotDemarcateInvalidXml"
+                     , "saveXmlDataOnly"
+                     , "useXSLTWhenSaving"
+                     , "saveThroughXslt"
+                     , "showXMLTags"
+                     , "alwaysMergeEmptyNamespace"
+                     , "updateFields"
+                     , "hdrShapeDefaults"
+                     , "footnotePr"
                      , "compat"
-                     ]
-  settingsEntry <- copyChildren refArchive distArchive settingsPath epochtime settingsList
+                     , "docVars"
+                     , "rsids"
+                     , "attachedSchema"
+                     , "themeFontLang"
+                     , "clrSchemeMapping"
+                     , "doNotIncludeSubdocsInStats"
+                     , "doNotAutoCompressPictures"
+                     , "forceUpgrade"
+                     , "captions"
+                     , "readModeInkLockDown"
+                     , "smartTagType"
+                     , "shapeDefaults"
+                     , "doNotEmbedSmartTags"
+                     , "decimalSymbol"
+                     , "listSeparator" ]
 
   let entryFromArchive arch path =
          maybe (throwError $ PandocSomeError
@@ -536,6 +669,11 @@ writeDocx opts doc = do
   docPropsAppEntry <- entryFromArchive refArchive "docProps/app.xml"
   themeEntry <- entryFromArchive refArchive "word/theme/theme1.xml"
   fontTableEntry <- entryFromArchive refArchive "word/fontTable.xml"
+  let fontTableRelsEntries = maybeToList $
+       findEntryByPath "word/_rels/fontTable.xml.rels" refArchive
+  let fontEntries = [entry | entry <- zEntries refArchive
+                           , "word/fonts/" `isPrefixOf` (eRelativePath entry)]
+                        -- or parse fontTable.xml.rels?
   webSettingsEntry <- entryFromArchive refArchive "word/webSettings.xml"
   headerFooterEntries <- mapM (entryFromArchive refArchive . ("word/" ++)) $
                          mapMaybe (fmap T.unpack . extractTarget)
@@ -555,7 +693,9 @@ writeDocx opts doc = do
                   commentsEntry :
                   docPropsEntry : docPropsAppEntry : customPropsEntry :
                   themeEntry :
-                  fontTableEntry : settingsEntry : webSettingsEntry :
+                  settingsEntry : webSettingsEntry :
+                  fontTableEntry :
+                  fontTableRelsEntries ++ fontEntries ++
                   imageEntries ++ headerFooterEntries ++
                   miscRelEntries ++ otherMediaEntries
   return $ fromArchive archive
@@ -592,14 +732,14 @@ styleToOpenXml sm style =
                              [ mknode "w:name" [("w:val", tshow toktype)] ()
                              , mknode "w:basedOn" [("w:val","VerbatimChar")] ()
                              , mknode "w:rPr" [] $
-                               [ mknode "w:color" [("w:val", tokCol toktype)] ()
-                                 | tokCol toktype /= "auto" ] ++
-                               [ mknode "w:shd" [("w:val","clear")
-                                                ,("w:fill",tokBg toktype)] ()
-                                 | tokBg toktype /= "auto" ] ++
                                [ mknode "w:b" [] () | tokFeature tokenBold toktype ] ++
                                [ mknode "w:i" [] () | tokFeature tokenItalic toktype ] ++
-                               [ mknode "w:u" [] () | tokFeature tokenUnderline toktype ]
+                               [ mknode "w:color" [("w:val", tokCol toktype)] ()
+                                 | tokCol toktype /= "auto" ] ++
+                               [ mknode "w:u" [] () | tokFeature tokenUnderline toktype ] ++
+                               [ mknode "w:shd" [("w:val","clear")
+                                                ,("w:fill",tokBg toktype)] ()
+                                 | tokBg toktype /= "auto" ]
                              ]
         tokStyles = tokenStyles style
         tokFeature f toktype = maybe False f $ M.lookup toktype tokStyles
@@ -627,17 +767,17 @@ copyChildren :: (PandocMonad m)
 copyChildren refArchive distArchive path timestamp elNames = do
   ref  <- parseXml refArchive distArchive path
   dist <- parseXml distArchive distArchive path
-  let elsToCopy =
-        map cleanElem $ filterChildrenName (\e -> qName e `elem` elNames) ref
-  let elsToKeep =
-        [e | Elem e <- elContent dist, not (any (hasSameNameAs e) elsToCopy)]
-  return $ toEntry path timestamp $ renderXml dist{
-      elContent = map Elem elsToKeep ++ map Elem elsToCopy
-    }
+  els <- foldM (addEl ref dist) [] (reverse elNames)
+  return $ toEntry path timestamp
+         $ renderXml dist{ elContent = map cleanElem els }
   where
-    hasSameNameAs (Element {elName = n1}) (Element {elName = n2}) =
-      qName n1 == qName n2
-    cleanElem el@Element{elName=name} = el{elName=name{qURI=Nothing}}
+    addEl ref dist els name =
+      case filterChildName (hasName name) ref `mplus`
+             filterChildName (hasName name) dist of
+        Just el -> pure (el : els)
+        Nothing -> pure els
+    hasName name = (== name) . qName
+    cleanElem el@Element{elName=name} = Elem el{elName=name{qURI=Nothing}}
 
 -- this is the lowest number used for a list numId
 baseListId :: Int
@@ -666,7 +806,7 @@ mkNum marker numid =
 mkAbstractNum :: ListMarker -> Element
 mkAbstractNum marker =
   mknode "w:abstractNum" [("w:abstractNumId",listMarkerToId marker)]
-    $ mknode "w:nsid" [("w:val", "A" <> listMarkerToId marker)] ()
+    $ mknode "w:nsid" [("w:val", T.justifyRight 8 '0' ("A" <> listMarkerToId marker))] ()
     : mknode "w:multiLevelType" [("w:val","multilevel")] ()
     : map (mkLvl marker)
       [0..maxListLevel]
@@ -768,7 +908,11 @@ writeOpenXML opts (Pandoc meta blocks) = do
   abstract <- if null abstract'
                  then return []
                  else do
-                   abstractTitle <- translateTerm Abstract
+                   abstractTitle <- case lookupMeta "abstract-title" meta of
+                       Just (MetaBlocks bs)   -> pure $ stringify bs
+                       Just (MetaInlines ils) -> pure $ stringify ils
+                       Just (MetaString s)    -> pure s
+                       _                      -> translateTerm Abstract
                    abstractTit <- withParaPropM (pStyleM "AbstractTitle") $
                                    blocksToOpenXML opts
                                      [Para [Str abstractTitle]]
@@ -794,8 +938,8 @@ writeOpenXML opts (Pandoc meta blocks) = do
               , mknode "w:r" []
                 [ mknode "w:rPr" []
                   [ mknode "w:rStyle" [("w:val", "CommentReference")] ()
-                  , mknode "w:annotationRef" [] ()
                   ]
+                  , mknode "w:annotationRef" [] ()
                 ]
               ] ++ annotation
             ]
@@ -924,7 +1068,11 @@ blockToOpenXML' _ b@(RawBlock format str)
       report $ BlockNotRendered b
       return []
 blockToOpenXML' opts (BlockQuote blocks) = do
-  p <- withParaPropM (pStyleM "Block Text")
+  inNote <- asks envInNote
+  p <- withParaPropM (pStyleM
+                       (if inNote
+                           then "Footnote Block Text"
+                           else "Block Text"))
        $ blocksToOpenXML opts blocks
   setFirstPara
   return p
@@ -1043,18 +1191,17 @@ toFigureTable opts blks = do
              map blockToCell blks
   let tbl = mknode "w:tbl" []
         ( mknode "w:tblPr" []
-          ( mknode "w:tblStyle" [("w:val","FigureTable")] () :
-            mknode "w:tblW" [ ("w:type", "auto"), ("w:w", "0") ] () :
+          [ mknode "w:tblStyle" [("w:val","FigureTable")] (),
+            mknode "w:tblW" [ ("w:type", "auto"), ("w:w", "0") ] (),
+            mknode "w:jc" [("w:val","center")] (),
             mknode "w:tblLook" [ ("w:firstRow", "0")
                                , ("w:lastRow", "0")
                                , ("w:firstColumn", "0")
                                , ("w:lastColumn", "0")
-                               ] () :
-            mknode "w:jc" [("w:val","center")] () :
-            []
-          )
+                               ] ()
+          ]
           : mknode "w:tblGrid" [] gridCols
-          : [tblBody]
+          : maybeToList tblBody
         )
   modify $ \s -> s { stInTable = False }
   return $ Elem tbl
@@ -1151,7 +1298,7 @@ getParaProps displayMathPara = do
   let listPr = [mknode "w:numPr" []
                 [ mknode "w:ilvl" [("w:val",tshow listLevel)] ()
                 , mknode "w:numId" [("w:val",tshow numid')] () ] | listLevel >= 0 && not displayMathPara]
-  return $ case listPr ++ squashProps props of
+  return $ case squashProps (EnvProps Nothing listPr <> props) of
                 [] -> []
                 ps -> [mknode "w:pPr" [] ps]
 
@@ -1270,12 +1417,12 @@ inlineToOpenXML' opts (Span (ident,classes,kvs) ils) = do
               langmod $ inlinesToOpenXML opts ils
   wrapBookmark ident contents
 inlineToOpenXML' opts (Strong lst) =
-  withTextProp (mknode "w:b" [] ()) $
   withTextProp (mknode "w:bCs" [] ()) $ -- needed for LTR, #6911
+  withTextProp (mknode "w:b" [] ()) $
   inlinesToOpenXML opts lst
 inlineToOpenXML' opts (Emph lst) =
-  withTextProp (mknode "w:i" [] ()) $
   withTextProp (mknode "w:iCs" [] ()) $  -- needed for LTR, #6911
+  withTextProp (mknode "w:i" [] ()) $
   inlinesToOpenXML opts lst
 inlineToOpenXML' opts (Underline lst) =
   withTextProp (mknode "w:u" [("w:val","single")] ()) $
@@ -1345,9 +1492,10 @@ inlineToOpenXML' opts (Note bs) = do
 
   contents <- local (\env -> env{ envListLevel = -1
                                 , envParaProperties = mempty
-                                , envTextProperties = mempty })
-              (withParaPropM (pStyleM "Footnote Text") $ blocksToOpenXML opts
-                $ insertNoteRef bs)
+                                , envTextProperties = mempty
+                                , envInNote = True })
+              (withParaPropM (pStyleM "Footnote Text") $
+               blocksToOpenXML opts $ insertNoteRef bs)
   let newnote = mknode "w:footnote" [("w:id", notenum)] contents
   modify $ \s -> s{ stFootnotes = newnote : notes }
   return [ Elem $ mknode "w:r" []
